<#ibiztemplate>
TARGET=PSSYSTEM
</#ibiztemplate>
<#assign hasMQProducer=false>
<#assign hasMQConsumer=false>
<#list sys.getAllPSDataEntities() as dataEntity>
    <#if dataEntity.getAllPSDEDataSyncs?? && dataEntity.getAllPSDEDataSyncs()??>
        <#list dataEntity.getAllPSDEDataSyncs() as dataSync>
            <#if dataSync.getInPSSysDataSyncAgent?? && dataSync.getInPSSysDataSyncAgent()??>
                <#assign hasMQConsumer=true>
            <#elseif dataSync.getOutPSSysDataSyncAgent?? && dataSync.getOutPSSysDataSyncAgent()??>
                <#assign hasMQProducer=true>
            </#if>
            <#if hasMQConsumer && hasMQProducer>
              <#break>
            </#if>
        </#list>
    </#if>
</#list>
<#if hasMQProducer || hasMQConsumer>
package ${pub.getPKGCodeName()}.core.util.config;

import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.consumer.DefaultMQPushConsumer;
import org.apache.rocketmq.client.exception.MQClientException;
import org.apache.rocketmq.client.producer.DefaultMQProducer;
import org.apache.rocketmq.common.consumer.ConsumeFromWhere;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Lazy;

@Slf4j
@Configuration
public class RocketMQConfig {

    <#if hasMQProducer>
    @Value("${r'${ibiz.rocketmq.producer.groupName:default}'}")
    private String groupName;
    @Value("${r'${ibiz.rocketmq.producer.namesrvAddr:127.0.0.1:9876}'}")
    private String namesrvAddr;
    // 消息最大值
    @Value("${r'${ibiz.rocketmq.producer.maxMessageSize:409600}'}")
    private Integer maxMessageSize;
    // 消息发送超时时间
    @Value("${r'${ibiz.rocketmq.producer.sendMsgTimeOut:3000}'}")
    private Integer sendMsgTimeOut;
    // 失败重试次数
    @Value("${r'${ibiz.rocketmq.producer.retryTimesWhenSendFailed:2}'}")
    private Integer retryTimesWhenSendFailed;

    /**
     * mq 生成者配置
     *
     * @return
     * @throws MQClientException
     */
    @Bean
    public DefaultMQProducer defaultProducer() throws MQClientException {
        log.info("rocketmq defaultProducer 正在创建---------------------------------------");
        DefaultMQProducer producer = new DefaultMQProducer(groupName);
        producer.setNamesrvAddr(namesrvAddr);
        producer.setVipChannelEnabled(false);
        producer.setMaxMessageSize(maxMessageSize);
        producer.setSendMsgTimeout(sendMsgTimeOut);
        producer.setRetryTimesWhenSendAsyncFailed(retryTimesWhenSendFailed);
        producer.start();
        log.info("rocketmq producer server 开启成功----------------------------------");
        return producer;
    }
    </#if>

    <#if hasMQConsumer>
    // 消费者线程数据量
    @Value("${r'${ibiz.rocketmq.consumer.consumeThreadMin:1}'}")
    private Integer consumeThreadMin;
    @Value("${r'${ibiz.rocketmq.consumer.consumeThreadMax:1}'}")
    private Integer consumeThreadMax;
    @Value("${r'${ibiz.rocketmq.consumer.consumeMessageBatchMaxSize:1}'}")
    private Integer consumeMessageBatchMaxSize;
    @Value("${r'${ibiz.rocketmq.topic:default}'}")
    private String topic;

    @Autowired
    @Lazy
    private RocketMQListenerProcessor listenerProcessor;

    /**
     * mq 消费者配置
     *
     * @return
     * @throws MQClientException
     */
    @Bean
    public DefaultMQPushConsumer defaultConsumer() {
        log.info("defaultConsumer 正在创建---------------------------------------");
        DefaultMQPushConsumer consumer = new DefaultMQPushConsumer(groupName);
        consumer.setNamesrvAddr(namesrvAddr);
        consumer.setConsumeThreadMin(consumeThreadMin);
        consumer.setConsumeThreadMax(consumeThreadMax);
        consumer.setConsumeMessageBatchMaxSize(consumeMessageBatchMaxSize);
        // 设置监听
        consumer.registerMessageListener(listenerProcessor);
        /**
         * 设置consumer第一次启动是从队列头部开始还是队列尾部开始
         * 如果不是第一次启动，那么按照上次消费的位置继续消费
         */
        consumer.setConsumeFromWhere(ConsumeFromWhere.CONSUME_FROM_FIRST_OFFSET);
        /**
         * 设置消费模型，集群还是广播，默认为集群
         */
//        consumer.setMessageModel(MessageModel.CLUSTERING);
        try {
            consumer.subscribe(topic, "${getSubscribeTags()}");
            consumer.start();
            log.info("rocketmq consumer 创建成功 groupName={}, topics={}, namesrvAddr={}", groupName, topic, namesrvAddr);
        } catch (MQClientException e) {
            log.error("rocketmq consumer 创建失败!" + e);
        }
        return consumer;
    }
    </#if>
}
</#if>

<#function getSubscribeTags>
    <#assign result="">
    <#list sys.getAllPSDataEntities() as dataEntity>
        <#if dataEntity.getAllPSDEDataSyncs?? && dataEntity.getAllPSDEDataSyncs()??>
            <#list dataEntity.getAllPSDEDataSyncs() as dataSync>
                <#if dataSync.getInPSSysDataSyncAgent?? && dataSync.getInPSSysDataSyncAgent()??>
                    <#if result!="">
                        <#assign result=result+" || ">
                    </#if>
                    <#assign result=result+dataSync.codeName?lower_case>
                </#if>
            </#list>
        </#if>
    </#list>
    <#return result>
</#function>